/*
 * Created by SharpDevelop.
 * User: spocke
 * Date: 2007-11-23
 * Time: 13:23
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Net;
using System.Web.Script.Serialization;
using System.Xml;

namespace Moxiecode.TinyMCE.SpellChecker {
	/// <summary>
	/// Description of GoogleSpellChecker.
	/// </summary>
	public class GoogleSpellChecker : ISpellChecker {


        public class GoogleResponse
        {
            public GoogleResponseResult result { get; set; }
        }
        public class GoogleResponseResult
        {
            public GoogleResponseSpellingCheckResponse spellingCheckResponse { get; set; }
        }
        public class GoogleResponseSpellingCheckResponse
        {
            public List<GoogleResponseMisspelling> misspellings { get; set; }
        }
        public class GoogleResponseMisspelling
        {
            public int charStart { get; set; }
            public int charLength { get; set; }
            public List<GoogleResponseSuggestion> suggestions { get; set; }
        }
        public class GoogleResponseSuggestion
        {
            public string suggestion { get; set; }
        }


		/// <summary>
		/// Description of GoogleSpellChecker.
		/// </summary>
		public GoogleSpellChecker() {
		}

		/// <summary>
		///  
		/// </summary>
		/// <param name="lang"></param>
		/// <param name="words"></param>
		/// <returns></returns>
		public string[] CheckWords(string lang, string[] words) {
			ArrayList misspelledWords = new ArrayList();
			XmlDocument doc = new XmlDocument();
			string result, wordsStr;

			// Send request to google
			wordsStr = String.Join(" ", words);
			result = this.SendRequest(lang, wordsStr);

            // Parse json response
            var js = new JavaScriptSerializer();
            var resultData = js.Deserialize<GoogleResponse>(result);

            // Get list of misspelled words
            if (resultData.result != null && resultData.result.spellingCheckResponse != null)
            {
                foreach (GoogleResponseMisspelling misspelling in resultData.result.spellingCheckResponse.misspellings)
                {
                    misspelledWords.Add(wordsStr.Substring(misspelling.charStart, misspelling.charLength));
                }
            }

            return (string[]) misspelledWords.ToArray(typeof(string));
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="lang"></param>
		/// <param name="word"></param>
		/// <returns></returns>
		public string[] GetSuggestions(string lang, string word) {
			ArrayList suggestions = new ArrayList();
			XmlDocument doc = new XmlDocument();
			string result;

			// Send request to google
			result = this.SendRequest(lang, word);

            // Parse json response
            var js = new JavaScriptSerializer();
            var resultData = js.Deserialize<GoogleResponse>(result);

            // Get list of suggestions
            if (resultData.result != null && resultData.result.spellingCheckResponse != null)
            {
                foreach (GoogleResponseMisspelling misspelling in resultData.result.spellingCheckResponse.misspellings)
                {
                    foreach (GoogleResponseSuggestion suggestion in misspelling.suggestions)
                    {
                        suggestions.Add(suggestion.suggestion);
                    }
                }
            }

			return (string[]) suggestions.ToArray(typeof(string));
		}

		#region private

		private string SendRequest(string lang, string data) {
			string server = "www.googleapis.com";
			string port = "443";
            string path = "/rpc";
			string url = "https://" + server + ":" + port + path;
            var requestData = new Dictionary<string, object>();
            var requestParams = new Dictionary<string, object>();
            requestParams.Add("language", lang);
            requestParams.Add("text", data);
            requestParams.Add("key", "AIzaSyCLlKc60a3z7lo8deV-hAyDU7rHYgL4HZg");
            requestData.Add("method", "spelling.check");
            requestData.Add("apiVersion", "v2");
            requestData.Add("params", requestParams);
            var js = new JavaScriptSerializer();
            string json = js.Serialize(requestData);
			string outData;
			StreamReader resStream = null;
			HttpWebResponse res = null;
			Stream reqStream = null;

			try {
				HttpWebRequest req = (HttpWebRequest) WebRequest.Create(url);
				req.KeepAlive = true;
				req.Method = "POST";
				req.ContentType = "application/json";
				req.ContentLength = json.Length;

				// Google-specific headers
				WebHeaderCollection reqHeaders = req.Headers;
				reqStream = req.GetRequestStream();

				ASCIIEncoding encoding = new ASCIIEncoding();
				byte[] jsonData = encoding.GetBytes(json);
				reqStream.Write(jsonData, 0, jsonData.Length);

				res = (HttpWebResponse) req.GetResponse();
				resStream = new StreamReader(res.GetResponseStream());
				outData = resStream.ReadToEnd();
			} finally {
				if (reqStream != null)
					reqStream.Close();

				if (resStream != null)
					resStream.Close();

				if (res != null)
					res.Close();
			}
				
			return outData;
		}

		private string EntityEncode(string str) {
			StringBuilder buff = new StringBuilder();

			for (int i=0; i<str.Length; i++) {
				char chr = str[i];

				if (chr > 127) {
					buff.Append("&#");
					buff.Append((int) chr);
					buff.Append(';');
				} else
					buff.Append(chr);
			}

			return buff.ToString();
		}
		
		#endregion
	}
}
